#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include <iso/limits_iso.h>
#include "CuTest.h"
#include "../triangle.h"
#include <sys/types.h>

#ifndef INT_MAX
#define INT_MIN         (-2147483647-1) /* min value of an "int" */
#define INT_MAX         2147483647      /* max value of an "int" */
#endif
/*-------------------------------------------------------------------------*
 * Demonstrates buttom-up testing of the triangle classification example 
 * Using Cutest test framework
 *-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*
 * Helper functions
 *-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*
 * classifyTriangle Test
 *-------------------------------------------------------------------------*/

void TestTriangleRectangular(CuTest* tc)
{
  CuAssertIntEquals(tc, rectangularTriangle, classifyTriangle(13,12,5) );
}

void TestTriangleRectangularDemoFail(CuTest* tc)
{ //for demo purposes demonstrate a testcase that fails (wrong expected output)
  CuAssertIntEquals(tc, rectangularTriangle, classifyTriangle(13,12,6) );
}

void TestTriangleScalene(CuTest* tc)
{
  CuAssertIntEquals(tc, scaleneTriangle, classifyTriangle(15,10,5) );
}
void TestTriangleEquilateral(CuTest* tc)
{
  CuAssertIntEquals(tc, equilateralTriangle, classifyTriangle(INT_MAX,INT_MAX,INT_MAX) );
}
void TestTriangleIsosceles(CuTest* tc)
{
  CuAssertIntEquals(tc, isoscelesTriangle, classifyTriangle(1,1,2) );
}

void TestTriangleInvalidZeroLength(CuTest* tc)
{
  CuAssertIntEquals(tc, invalidTriangle, classifyTriangle(100,0,10) );
}

void TestTriangleInvalidNegativeLength(CuTest* tc)
{
  CuAssertIntEquals(tc, invalidTriangle, classifyTriangle(100,50,-1) );
}


CuSuite* TestTriangleGetSuite(void)
{
	CuSuite* suite = CuSuiteNew();

	SUITE_ADD_TEST(suite, TestTriangleRectangular);
	SUITE_ADD_TEST(suite, TestTriangleScalene);
	SUITE_ADD_TEST(suite, TestTriangleIsosceles);
	SUITE_ADD_TEST(suite, TestTriangleEquilateral);
	SUITE_ADD_TEST(suite, TestTriangleInvalidNegativeLength);
	SUITE_ADD_TEST(suite, TestTriangleInvalidZeroLength);
	SUITE_ADD_TEST(suite, TestTriangleRectangularDemoFail);
	return suite;
}

/*-------------------------------------------------------------------------*
 * Helper functions
 *-------------------------------------------------------------------------*/
#define CuAssertBoolEquals(tc,ex,ac)  CuAssertIntEquals(tc,ex,ac) 
#define CuAssertBoolEquals_Msg(tc,ms,ex,ac) CuAssertIntEquals_Msg(tc,ms,ex,ac)

#define CuAssertConvertMatch(tc,exs,exv,acs,acv)           CuAssertConvertMatch_LineMsg((tc),__FILE__,__LINE__,NULL,(exs),(exv),(acs),(acv))
#define CuAssertConvertMatch_Msg(tc,ms,exs,exv,acs,acv)    CuAssertConvertMatch_LineMsg((tc),__FILE__,__LINE__,(ms),(exs),(exv),(acs),(acv))

void CuAssertConvertMatch_LineMsg(CuTest* tc, const char* file, 
				   int line, const char* message, 
				   boolean_t expected_success, long expected_value,
				   boolean_t actual_success, int actual_value)
{
	char buf[STRING_MAX];
	if (expected_success == B_TRUE) {
	  if((actual_success==B_TRUE) && (expected_value==actual_value)) 
	    return;
	}
	else
	  if(expected_success==actual_success) //expected_success==B_FALSE
	    return; //actual values need not match

	sprintf(buf, "expected <success=%d,value=%d> but was <success=%d,value=%d>", expected_success, expected_value, actual_success,actual_value);
	CuFail_Line(tc, file, line, message, buf);
}

/*-------------------------------------------------------------------------*
 * stringToInt Test
 *-------------------------------------------------------------------------*/
extern boolean_t stringToInt(char* s, int *a);

void TestStringToIntPositiveZero(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("1",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, 1, actual_success,actual_val );
}
void TestStringToIntPositiveSmall(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("+0",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, 0, actual_success,actual_val );
}

void TestStringToIntPositiveMedium(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("999",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, 999, actual_success,actual_val );
}
void TestStringToIntPositiveLarge(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("999999999",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, 999999999, actual_success,actual_val );
}

void TestStringToIntNegativeZero(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("-0",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, 0, actual_success,actual_val );
}

void TestStringToIntNegativeSmall(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("-1",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, -1, actual_success,actual_val );
}
void TestStringToIntNegativeMedium(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("-999",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, -999, actual_success,actual_val );
}
void TestStringToIntNegativeLarge(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("-999999999",&actual_val);
  
  CuAssertConvertMatch(tc, B_TRUE, -999999999, actual_success,actual_val );
}

void TestStringToIntTooLong(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("-9999999999999",&actual_val);
  
  CuAssertConvertMatch(tc, B_FALSE, 0, actual_success,actual_val );
}
void TestStringToIntTooShort(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("-",&actual_val);
  
  CuAssertConvertMatch(tc, B_FALSE, 0, actual_success,actual_val );
}
void TestStringToIntEmptyString(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("",&actual_val);
  
  CuAssertConvertMatch(tc, B_FALSE, 0, actual_success,actual_val );
}

void TestStringToIntNAN(CuTest* tc)
{
  int actual_val;
  boolean_t actual_success=stringToInt("kurt",&actual_val);
  
  CuAssertConvertMatch(tc, B_FALSE, 0, actual_success,actual_val );
}


CuSuite* TestStringToIntGetSuite(void)
{
	CuSuite* suite = CuSuiteNew();

	SUITE_ADD_TEST(suite, TestStringToIntPositiveZero);
	SUITE_ADD_TEST(suite,  TestStringToIntPositiveSmall);
	SUITE_ADD_TEST(suite,  TestStringToIntPositiveMedium);
	SUITE_ADD_TEST(suite,  TestStringToIntPositiveLarge);
	SUITE_ADD_TEST(suite,  TestStringToIntNegativeZero);
	SUITE_ADD_TEST(suite,  TestStringToIntNegativeSmall);
	SUITE_ADD_TEST(suite,  TestStringToIntNegativeMedium);
	SUITE_ADD_TEST(suite,  TestStringToIntNegativeLarge);
	SUITE_ADD_TEST(suite,  TestStringToIntTooLong);
	SUITE_ADD_TEST(suite,  TestStringToIntTooShort);
	SUITE_ADD_TEST(suite,  TestStringToIntEmptyString);
	SUITE_ADD_TEST(suite,  TestStringToIntNAN);

	return suite;
}


/*-------------------------------------------------------------------------*
 * checkTriangle Test
 *-------------------------------------------------------------------------*/
extern char* checkTriangle(int argc, char ** argv);
//argc assumed to correctly indicate number of arguments in argv vector
//since this is normally done by the OS and C startup code (assumed correct).
/*
  "Invalid Triangle",
  "Rectangular Triangle",
  "Equilateral Triangle",
  "Isosceles Triangle",
  "Scalene Triangle",
*/
char * programName="testTriangle";

void TestCheckTriangleRectangular(CuTest* tc) {
  char*argv[4]=  {programName,"13","12","5"};
   CuAssertStrEquals(tc, "Rectangular Triangle", 
		     checkTriangle(4,argv));
}
void TestCheckTriangleScalene(CuTest* tc) {
  char*argv[4]=  {programName,"15","10","5"};
   CuAssertStrEquals(tc, "Scalene Triangle", 
		     checkTriangle(4,argv));
}

void TestCheckTriangleEquilateral(CuTest* tc) {
  char*argv[4]=  {programName,"999999999","999999999","999999999"};
   CuAssertStrEquals(tc, "Equilateral Triangle", 
		     checkTriangle(4,argv));
}

void TestCheckTriangleIsosceles(CuTest* tc) {
  char*argv[4]=  {programName,"1","1","2"};
   CuAssertStrEquals(tc, "Isosceles Triangle", 
		     checkTriangle(4,argv));
}

void TestCheckTriangleInvalidZeroLength(CuTest* tc) {
  char*argv[4]=  {programName,"100","100","0"};
   CuAssertStrEquals(tc, "Invalid Triangle", 
		     checkTriangle(4,argv));
}

void TestCheckTriangleInvalidNegativeLength(CuTest* tc) {
  char*argv[4]=  {programName,"100","100","-1"};
   CuAssertStrEquals(tc, "Invalid Triangle", 
		     checkTriangle(4,argv));
}

void TestCheckTriangleNAN3(CuTest* tc) {
  char*argv[4]=  {programName,"100","100","peter"};
   CuAssertStrEquals(tc, "Error in third argument\n", 
		     checkTriangle(4,argv));
}
void TestCheckTriangleTooFewArgs(CuTest* tc) {
  char*argv[3]=  {programName,"100","peter"};
   CuAssertStrEquals(tc, "Incorrect Number of Arguments\n", 
		     checkTriangle(2,argv));
}
void TestCheckTriangleTooManyArgs(CuTest* tc) {
  char*argv[5]=  {programName,"100","peter","100",""};
   CuAssertStrEquals(tc, "Incorrect Number of Arguments\n", 
		     checkTriangle(5,argv));
}

CuSuite* TestCheckTriangletGetSuite(void)
{
	CuSuite* suite = CuSuiteNew();

	SUITE_ADD_TEST(suite, TestCheckTriangleRectangular);
	SUITE_ADD_TEST(suite, TestCheckTriangleScalene);
	SUITE_ADD_TEST(suite, TestCheckTriangleEquilateral);
	SUITE_ADD_TEST(suite, TestCheckTriangleIsosceles);
	SUITE_ADD_TEST(suite, TestCheckTriangleInvalidZeroLength);
	SUITE_ADD_TEST(suite, TestCheckTriangleInvalidNegativeLength);
	SUITE_ADD_TEST(suite, TestCheckTriangleNAN3);
	SUITE_ADD_TEST(suite, TestCheckTriangleTooFewArgs);
	SUITE_ADD_TEST(suite, TestCheckTriangleTooManyArgs);
	
	return suite;
}

/*-------------------------------------------------------------------------*
 * These test demonstrates how to test a function that makes calls to
 * other functions in ISOLATION. The technique is to replace the
 * called function with a stub whose return value/side effect is
 * controlled by the test-driver.
 * compile with -D  STUB_classifyTriangle
 *-------------------------------------------------------------------------*/
#ifdef STUB_classifyTriangle
static TriangleKind STUB_classifyTriangleReturn_kind=invalidTriangle;	
TriangleKind  classifyTriangle(int a, int b, int c) {
  return STUB_classifyTriangleReturn_kind;
}


extern char* checkTriangle(int argc, char ** argv);
//argc assumed to correctly indicate number of arguments in argv vector
//since this is normally done by the OS and C startup code (assumed correct).
/*
  "Invalid Triangle",
  "Rectangular Triangle",
  "Equilateral Triangle",
  "Isosceles Triangle",
  "Scalene Triangle",
*/
//char * programName="testTriangle";

void TestStubCheckTriangleRectangular(CuTest* tc) {
  char*argv[4]=  {programName,"13","12","5"};
  STUB_classifyTriangleReturn_kind=rectangularTriangle;
  CuAssertStrEquals(tc, "Rectangular Triangle", 
		    checkTriangle(4,argv));
}
void TestStubCheckTriangleScalene(CuTest* tc) {
  char*argv[4]=  {programName,"15","10","5"};
  STUB_classifyTriangleReturn_kind=scaleneTriangle;
   CuAssertStrEquals(tc, "Scalene Triangle", 
		     checkTriangle(4,argv));
}

void TestStubCheckTriangleEquilateral(CuTest* tc) {
  char*argv[4]=  {programName,"999999999","999999999","999999999"};
  STUB_classifyTriangleReturn_kind=equilateralTriangle;
   CuAssertStrEquals(tc, "Equilateral Triangle", 
		     checkTriangle(4,argv));
}

void TestStubCheckTriangleIsosceles(CuTest* tc) {
  char*argv[4]=  {programName,"1","1","2"};
STUB_classifyTriangleReturn_kind=isoscelesTriangle;
   CuAssertStrEquals(tc, "Isosceles Triangle", 
		     checkTriangle(4,argv));
}

void TestStubCheckTriangleInvalidZeroLength(CuTest* tc) {
  char*argv[4]=  {programName,"100","100","0"};
  STUB_classifyTriangleReturn_kind=invalidTriangle;
   CuAssertStrEquals(tc, "Invalid Triangle", 
		     checkTriangle(4,argv));
}

void TestStubCheckTriangleInvalidNegativeLength(CuTest* tc) {
  char*argv[4]=  {programName,"100","100","-1"};
  STUB_classifyTriangleReturn_kind=invalidTriangle;
   CuAssertStrEquals(tc, "Invalid Triangle", 
		     checkTriangle(4,argv));
}

void TestStubCheckTriangleNAN3(CuTest* tc) {
  char*argv[4]=  {programName,"100","100","peter"};

  //return value should not be used: set to value that would result
  //in errornous output from the tested procedure.
  
 STUB_classifyTriangleReturn_kind=-1;
   CuAssertStrEquals(tc, "Error in third argument\n", 
		     checkTriangle(4,argv));
}
void TestStubCheckTriangleTooFewArgs(CuTest* tc) {
  char*argv[3]=  {programName,"100","peter"};
 STUB_classifyTriangleReturn_kind=-1;
   CuAssertStrEquals(tc, "Incorrect Number of Arguments\n", 
		     checkTriangle(2,argv));
}
void TestStubCheckTriangleTooManyArgs(CuTest* tc) {
  char*argv[5]=  {programName,"100","peter","100",""};
 STUB_classifyTriangleReturn_kind=-1;
   CuAssertStrEquals(tc, "Incorrect Number of Arguments\n", 
		     checkTriangle(5,argv));
}

CuSuite* TestStubCheckTriangletGetSuite(void)
{
	CuSuite* suite = CuSuiteNew();

	SUITE_ADD_TEST(suite, TestStubCheckTriangleRectangular);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleScalene);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleEquilateral);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleIsosceles);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleInvalidZeroLength);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleInvalidNegativeLength);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleNAN3);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleTooFewArgs);
	SUITE_ADD_TEST(suite, TestStubCheckTriangleTooManyArgs);
	
	return suite;
}
#endif 
